/*=============================================================================
 *     Copyright Texas Instruments 2002. All Rights Reserved.
 */


/*=============================================================================
    Description:
    This test is derived from the TestCase base class.

    This test case verifies that edge cases concerning the size of entries
    and the sizes of primitives are handled well.


    Args:
    logWriter        -  A writer object that is used to log any messages.
                        User can either pass in a writer object or null.

    warningWriter    -  A writer object that is used to log any warning messages.
                        User can either pass in a writer object or null.

    errorWriter      -  A writer object that is used to log any error messages
                        User can either pass in a writer object or null.

    Returns:
    n/a

 */
public function TestEntrySizeEdgeCase( logWriter, warningWriter, errorWriter )
  extends pkg.tf.TestCase( "esf.age.TestEntrySizeEdgeCase", logWriter, warningWriter, errorWriter)
{
  /****************************************************************
   * Global variable initionalizations.
   ****************************************************************/
  const var scriptEhId = 0x101;

  const var rctx = services["esf routing"].getRoutingContext();


  //====================================================================================
  /**
   * Overrides the runTest() method in TestCase base class.
   *
   */
  public function runTest()
  {
    var prim;

    logInfo( "Beginning Age Event Handler Unit Test- TestEntrySizeEdgeCase" );
    const var p = pkg.esf.age.util;
    var phase;
    const var preallocSize = 20;
    const var preallocCount = 8;
    const var refreshPeriod = 0;


    // TODO make sure that nothing else is using this event handler



    // Make the system quiet

    phase = "Init:";
    if (!p.unitTestStart( rctx, scriptEhId, phase, preallocCount, preallocSize, 0 )) return;

    phase = "TestLastEntryInPrim:";
    const var osEntries = [];
    var osEntryCount = 0;
    const var attemptedEntries = [];
    const var attemptedEntriesCount = 10;
    const var attemptedEntryErrorCodes = [];
    const var entryHeaderSize = 4;  /* Could change; unlikely */

    for (var i = 0; i < attemptedEntriesCount; ++i)
    {
      const var entryLength =  preallocSize - 1 + ( i % 3 );
      attemptedEntries[ i ] = new p.OSEntry();
      attemptedEntries[ i ].randomize( 0, 0, entryLength - entryHeaderSize, i + 123);
      const var tag = 0xAA00 + (i % 0xFF);
      attemptedEntries[ i ].setTag( tag );

      logInfo( phase + " entry for index " + i +" is " + attemptedEntries[ i ].toString() );
      attemptedEntryErrorCodes[ i ] =
          ti.th.prim.Age_AgeEh_InsertEntryResult.
            AGE_AGEEH_INSERT_ENTRY_FAIL_ENTRY_TOO_BIG;
      logInfo( phase + " index " + i + "- entryLength:" + entryLength
               + " and primSize:" + preallocSize );

      if (entryLength <= preallocSize)
      {
        osEntries[ osEntryCount ] = attemptedEntries[ i ];
        osEntryCount++;
        attemptedEntryErrorCodes[ i ] =
          ti.th.prim.Age_AgeEh_InsertEntryResult.AGE_AGEEH_INSERT_ENTRY_SUCCESS;
        logInfo( phase + "entryLength:" + entryLength
                 + " and primSize:" + preallocSize + " INCLUDED " );
      }
    }

    if (osEntryCount >= attemptedEntriesCount)
    {
      logError( phase + "Test design issue: osEntryCount:" + osEntryCount
                + " must be < attemptedEntriesCount:" + attemptedEntriesCount );
    }

    if (osEntryCount == 0)
    {
      logError( phase + "Test design issue: osEntryCount:"
                + osEntryCount + " must be > 0 " );
    }

    for (var i = 0; i < attemptedEntries.length(); ++i)
    {
      if (!p.writeBasicEntry(
            rctx
          , scriptEhId
          , attemptedEntries[ i ]
          , attemptedEntryErrorCodes[ i ]
        ) ) logError( phase + "Entry write fail." );
    }


    phase = "VerifyOnlyOsEntryWritten:";
    if (!p.forceRelinquish( rctx, scriptEhId ) ) logError( phase + "Relinquish Fail." );
    var trap = p.attachTrap( rctx, scriptEhId );
    if (!p.triggerBufferService( rctx, scriptEhId) ) logError( phase + "Buffer Service Fail." );
    var decoder = pkg.esf.age.util.acquireEntryDecoder( logInfo, logWarning, logError );
    decoder.setTestReporter( new function () extends ti.test.TestReporter() {
        public function report( string ) {
          logInfo( "TR: " + string );
        }
      } ()
    );
    if (!p.forceRelinquish( rctx, scriptEhId ) ) logError( phase + "Relinquish Fail." );
    p.detachDecodeTrap( rctx, trap, decoder );
    const var decodedEntryCount = decoder.entryCount();
    logInfo( "Decoded entry count was " + decodedEntryCount
             + " and osEntryCount is " + osEntryCount );
    if (decodedEntryCount != osEntryCount)
    {
      logError( phase + "Expected " + osEntryCount
                + " entries but found " + decoder.entryCount() );
    }
    for (var i = 0; i < decodedEntryCount; ++i) {
      logInfo( "Inspecting entry i= " + i + ": Expected is " + osEntries[ i ].toString() );
      const var decodedEntry = decoder.get();
      if (!p.verifyBasicEntry( decodedEntry, osEntries[ i ] ) )
      {
        logError( phase + "Entry verify fail." );
      }
    }
    while( decoder.entryCount() > 0) {
      const var unhandled = decoder.get();
      logError( phase + "Unhandled Entry!" + unhandled );
    }
    for (
          var lostEntryIndex = decodedEntryCount
        ; lostEntryIndex < osEntryCount
        ; ++lostEntryIndex
        )
    {
      logError(" Lost the entry " + osEntries[ lostEntryIndex ] );
    }

    phase = "Cleanup:";
    p.unitTestEnd( rctx, scriptEhId, phase );

  }

}




